"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.oForEach = oForEach;
exports.oMap = oMap;
exports.oMerge = oMerge;
exports.oReduce = oReduce;
exports.shallowEqual = shallowEqual;

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function oForEach(subject, operation) {
  for (var _key in subject) {
    if (subject.hasOwnProperty(_key)) {
      operation(subject[_key], _key);
    }
  }
}

function oMap(subject, mapper) {
  var result = {};
  oForEach(subject, function (value, key) {
    result[key] = mapper(value, key);
  });
  return result;
}

function oMerge(subject, updates) {
  oForEach(updates, function (value, key) {
    subject[key] = value;
  });
  return subject;
}

function oReduce(subject, reducer, initialAcc) {
  var result = initialAcc;
  oForEach(subject, function (value, key) {
    result = reducer(result, value, key);
  });
  return result;
}

function isImmutable(obj) {
  return obj && typeof obj.hashCode === 'function' && typeof obj.equals === 'function';
}

function _compareArrays(arr1, arr2) {
  if (!arr1 || !arr2) return false;

  if (arr1.length !== arr2.length) {
    return false;
  }

  if (!arr1.length) {
    return true;
  }

  for (var idx in arr1) {
    // this makes it _technically_ not a shallow equal for array values,
    // but for stores that return arrays of objects, the object identities
    // will likely not be equal so we should actually inspect the objects
    // to see if they're equal
    if (!shallowEqual(arr1[idx], arr2[idx])) {
      return false;
    }
  }

  return true;
}

function shallowEqual(obj1, obj2) {
  if (obj1 === obj2) {
    return true;
  } // Special handling for FB Immutables, as they must be handled specially.
  // While this technically means this is deep equality for Immutables,
  // it's better to have a more specific check than an entirely incorrect one.


  if (isImmutable(obj1) && isImmutable(obj2)) {
    return obj1.equals(obj2);
  }

  if (_typeof(obj1) !== _typeof(obj2)) {
    return false;
  } // would have passed === check if both were same falsy value


  if (!obj1 || !obj2) {
    return false;
  }

  if (_typeof(obj1) !== 'object') {
    return false;
  }

  if (Object.keys(obj1).length !== Object.keys(obj2).length) {
    return false;
  }

  for (var property in obj1) {
    if (obj1.hasOwnProperty(property) && !obj2.hasOwnProperty(property)) {
      return false;
    }

    if (!obj1.hasOwnProperty(property) && obj2.hasOwnProperty(property)) {
      return false;
    }

    if (obj1.hasOwnProperty(property) && obj2.hasOwnProperty(property)) {
      if (isImmutable(obj1[property]) && isImmutable(obj2[property])) {
        if (!obj1[property].equals(obj2[property])) {
          return false;
        }
      } else if (Array.isArray(obj1[property]) || Array.isArray(obj2[property])) {
        return _compareArrays(obj1[property], obj2[property]);
      } else if (obj1[property] !== obj2[property]) {
        return false;
      }
    }
  }

  return true;
}